
#define PI 3.1415

//// UN-TWEAKABLES - AUTOMATICALLY-TRACKED TRANSFORMS ////////////////

float4x4 WorldITXf : WorldInverseTranspose < string UIWidget="None"; >;
float4x4 WvpXf : WorldViewProjection < string UIWidget="None"; >;
float4x4 WorldXf : World < string UIWidget="None"; >;
float4x4 ViewIXf : ViewInverse < string UIWidget="None"; >;

//// TWEAKABLE PARAMETERS ////////////////////

/// Point Lamp 0 ////////////
float3 Lamp0Pos : Position <
    string Object = "PointLight0";
    string UIName =  "Lamp 0 Position";
    string Space = "World";
> = {-0.5f,2.0f,1.25f};
float3 Lamp0Color : Specular <
    string UIName =  "Lamp 0";
    string Object = "Pointlight0";
    string UIWidget = "Color";
> = {1.0f,1.0f,1.0f};

/* Outer Radius*/
float rmax <
	string UIWidget = "slider";
	float  UIMin = 0.0;
	float  UIMax = 0.5;
	float  UIStep = 0.01;
	string UIName =  "Outer Radius";
> = 0.4;

/* Inner Radius*/
float rmin <
	string UIWidget = "slider";
	float  UIMin = 0.0;
	float  UIMax = 0.5;
	float  UIStep = 0.01;
	string UIName =  "Inner Radius";
> = 0.2;

/* Number of Stars X*/
float nStarsX <
	string UIWidget = "slider";
	float  UIMin = 1.0;
	float  UIMax = 20.0;
	float  UIStep = 1.0;
	string UIName =  "Number of Stars X";
> = 1.0;

/* Number of Stars Y*/
float nStarsY <
	string UIWidget = "slider";
	float  UIMin = 1.0;
	float  UIMax = 20.0;
	float  UIStep = 1.0;
	string UIName =  "Number of Stars Y";
> = 1.0;

/* Number of Sections*/
float npoints <
	string UIWidget = "slider";
	float  UIMin = 4.0;
	float  UIMax = 20.0;
	float  UIStep = 1.0;
	string UIName =  "Number of Sections";
> = 7.0;

/* Ambientes Licht */
float3 starColor <
	string UIName =  "Star Color";
	string UIWidget = "Color";
> = {0.9f, 1.0f, 0.5f};

/* Ambientes Licht */
float3 backColor <
	string UIName =  "Background Color";
	string UIWidget = "Color";
> = {0.5f, 0.5f, 0.1f};

/* Farbe des Sterns */
float3 AmbiColor : Ambient <
	string UIName =  "Ambient Lighting";
	string UIWidget = "Color";
> = {0.1f, 0.1f, 0.1f};

/* Farbe des Hintergrunds */
float3 SurfColor : DIFFUSE <
	string UIName =  "Surface Color";
	string UIWidget = "Color";
> = {0.8f, 0.8f, 1.0f};

/* Diffuse Intensität */
float Kd <
	string UIWidget = "slider";
	float  UIMin = 0.0;
	float  UIMax = 1.0;
	float  UIStep = 0.01;
	string UIName =  "Diffuse Intensity";
> = 0.5;

/* Spekulare Intensität */
float Ks <
	string UIWidget = "slider";
	float  UIMin = 0.0;
	float  UIMax = 1.0;
	float  UIStep = 0.01;
	string UIName =  "Specular Intensity";
> = 0.5;

/* Spekularer Exponent */
float s : SpecularPower <
	string UIWidget = "slider";
	float UIMin = 1.0;
	float UIMax = 1000.0;
	float UIStep = 10.0;
	string UIName = "Specular Power";
> = 30.0;

texture EnvMap : ENVIRONMENT <
    string ResourceName = "EnvironmentMapMS.dds";
    string UIName =  "Environment";
    string ResourceType = "Cube";
>;
samplerCUBE EnvMapSampler = sampler_state 
{
    texture = <EnvMap>;
    FILTER = MIN_MAG_MIP_LINEAR;
    AddressU  = Clamp;        
    AddressV  = Clamp;
    AddressW  = Clamp;
};

//////// STREAM DATA STRUCTURES ///////////

/* data from application vertex buffer */
struct appdata {
    float3 Position	: POSITION;
    float4 UV		: TEXCOORD0;
    float4 Normal	: NORMAL;
    float4 Tangent	: TANGENT0;
    float4 Binormal	: BINORMAL0;
};

/* data passed from vertex shader to pixel shader */
struct vertexOutput {
    float4 HPosition	: POSITION;
    float2 UV		: TEXCOORD0;
    // The following values are passed in "World" coordinates since
    //   it tends to be the most flexible and easy for handling
    //   reflections, sky lighting, and other "global" effects.
    float3 LightVec	: TEXCOORD1;
    float3 WorldNormal	: TEXCOORD2;
    float3 WorldTangent	: TEXCOORD3;
    float3 WorldBinormal : TEXCOORD4;
    float3 WorldView	: TEXCOORD5;
};
 
///////// VERTEX SHADING /////////////////////

/*********** Generic Vertex Shader ******/

vertexOutput std_VS(appdata IN) {
    vertexOutput OUT = (vertexOutput)0;
    OUT.WorldNormal = mul(IN.Normal,WorldITXf).xyz;
    OUT.WorldTangent = mul(IN.Tangent,WorldITXf).xyz;
    OUT.WorldBinormal = mul(IN.Binormal,WorldITXf).xyz;
    float4 Po = float4(IN.Position.xyz,1);
    float3 Pw = mul(Po,WorldXf).xyz;
    OUT.LightVec = (Lamp0Pos - Pw);
    OUT.UV = IN.UV.xy;
    OUT.WorldView = normalize(ViewIXf[3].xyz - Pw);
    OUT.HPosition = mul(Po,WvpXf);
    return OUT;
}

///////// PIXEL SHADING //////////////////////

// utility function
float modulo(float a, float b) {

    a -= floor(a/b)*b;
	if (a < 0) a+=b;
	return a;
}
 
float4 procstar_PS(vertexOutput IN) : COLOR {
	
  // how many stars should be visible in u- and v-direction?
  float mX = 1.0/nStarsX;
  float mY = 1.0/nStarsY;
  float2 uv = float2( modulo(IN.UV.x,mX)/mX, modulo(IN.UV.y,mY)/mY); 
  
  float  starangle = 2.0*PI/npoints;
  float3 p1 = rmax * float3(cos(0.0),         sin(0.0),         0.0);
  float3 p0 = rmin * float3(cos(starangle/2), sin(starangle/2), 0.0);

  float3 d0 = p1 - p0;

  float2 st = uv -float2(0.5,0.5);
  float angle = atan2(st.x,st.y);
  
  float r = sqrt(dot(st,st));

  float A = modulo(angle, starangle)/starangle;
  if (A >= 0.5) A = 1-A;

  float3 d1 = r * float3(cos(A), sin(A), 0) - p0;
  float in_out = step(0,cross(d0,d1).z);
  
  float4 color;
  
  float3 N = normalize(IN.WorldNormal);
  float3 L = normalize(IN.LightVec);
  float3 V = normalize(IN.WorldView);
  float3 H = normalize(L + V);
  float3 R = dot(V,N) * 2.0 * N - V;

  float4 envColor = texCUBE(EnvMapSampler,R);
  // Compute the diffuse term
  float diffuseLight = max(dot(L, N),0);
  float3 diffuse =  Kd * diffuseLight;

  // Compute the specular term
  float specularLight = pow(max(dot(H, N),0),s);
  float3 specular = Ks * specularLight;
  
  color.xyz = lerp(starColor*envColor,(diffuse + specular)*backColor,in_out);
  color.w = 1;

  return color;
}

///// TECHNIQUES /////////////////////////////

RasterizerState DisableCulling
{
    CullMode = NONE;
};

DepthStencilState DepthEnabling
{
	DepthEnable = TRUE;
};

BlendState DisableBlend
{
	BlendEnable[0] = FALSE;
};

technique10 Main10 <
	string Script = "Pass=p0;";
> {
    pass p0 <
	string Script = "Draw=geometry;";
    > {
        SetVertexShader( CompileShader( vs_4_0, std_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, procstar_PS() ) );
                
        SetRasterizerState(DisableCulling);       
		SetDepthStencilState(DepthEnabling, 0);
		SetBlendState(DisableBlend, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
    }
}


technique Main <
	string Script = "Pass=p0;";
> {
    pass p0 <
	string Script = "Draw=geometry;";
    > {
        VertexShader = compile vs_2_0 std_VS();
		ZEnable = true;
		ZWriteEnable = true;
		ZFunc = LessEqual;
		AlphaBlendEnable = false;
		CullMode = None;
        PixelShader = compile ps_2_a procstar_PS();
    }
}

/////////////////////////////////////// eof //
